/* NotifyMe 1.1 
 * Run this program after you log in.
 * It will read resource file ($HOME/.notifyrc by default) and then
 * will stay in background waiting for a specifing user to come.
 * If that happen you'll see small message on the screen.
 *
 * author: Michal Suszycki 	
 *				<mike@wizard.ae.krakow.pl>
 *				http://wizard.ae.krakow.pl/~mike
 */

#include <stdio.h>
#include <utmp.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>

#ifndef UTMP_FILE
#define UTMP_FILE "/var/run/utmp"
#endif

#ifndef WTMP_FILE
#define WTMP_FILE "/var/log/wtmp"
#endif

#ifndef UT_NAMESIZE
#define UT_NAMESIZE 8
#endif

#define MAX_USERS 64

char *rcfile = ".notifyrc";

/* 
 * save current state
 */
#define SAVE "\0337"

/* 
 * go to corner
 */
#define GO "\033[0;67H"

/* 
 * bold 
 */
#define BOLD "\033[1m"

/*
 * reset attributes to their defaults
 */
#define RESET "\033[0m"

/* 
 * restore most recently saved state
 */
#define RESTORE "\0338"

struct user
{
	char *name;
	int count;
};

static struct user users[MAX_USERS];

int read_conf(){
	FILE *f;
	char *env, buf[512];
	int i = 0;
	
	if (!(env = getenv("HOME"))){
		fprintf(stderr,"HOME is not set\n");
		exit(1);
	}
	snprintf(buf,strlen(env) + strlen(rcfile) + 2,"%s/%s",env,rcfile);
	
	if (!(f = fopen(buf,"r"))){
		perror("fopen");
		fprintf(stderr,"Cannot read resource file %s\n",buf);
		exit (1);
	}
	while (fscanf(f,"%s\n",buf) != EOF){
		if (i == MAX_USERS - 2) break;
		if (!(users[i].name = malloc(strlen(buf) + 1))){
			fprintf(stderr,"Cannot allocate memory.\n");
			exit (1);
		}
		strcpy(users[i++].name,buf);
	}
	users[i].name = 0;
	return (0);
}


void read_utmp()
{
	int fd, i, c;
	struct utmp ent;
	
	if (!(fd = open(UTMP_FILE,O_RDONLY))){
		perror("open");
		exit(0);
	}
	while(read(fd, &ent, sizeof ent) > 0){
		if (ent.ut_type != USER_PROCESS) continue;
		for (i = 0; users[i].name; i++)
			if (!strncmp(users[i].name,ent.ut_user,UT_NAMESIZE))
				users[i].count++;
	}
	close(fd);			
	for (i = 0, c = 0; users[i].name; i++)
		if(users[i].count){
			if (!c) printf("\nUsers currently on the machine:\n");
			printf("%-8.9s (%d time", users[i].name, users[i].count);
			printf("%s\n",(users[i].count==1)?")":"s)");
			c++;
	}
	if (c) printf("\n");

}
		

int main (int argc, char *argv[])
{
	int f;
	int i = 0, v, prnt;
	struct utmp buf;
	char tmp[128];
	
	if ((f = open(WTMP_FILE,O_RDONLY)) < 0){
		printf ("Cannot open " WTMP_FILE "\n");
		exit (0);
	}
	read_conf();
	read_utmp();

	lseek(f,0,SEEK_END);
	prnt = getppid();
	switch(v = fork()){
		case -1: fprintf(stderr,"Cannot fork.\n"); exit (-1);
		case 0 : setpgid(getpid(),prnt);break;
		default: exit (0);
	}

	while(1){
		while (!read(f,&buf,sizeof buf)) sleep(3);
		if (buf.ut_type != USER_PROCESS) continue;
		for(i = 0; users[i].name; i++)
			if (!strcmp(users[i].name,buf.ut_user)){
				printf(SAVE GO "        " RESTORE);
				sprintf(tmp,"\033[0;%dH",72-strlen(users[i].name));
				printf(SAVE "%s" BOLD,tmp);
				printf("%s arrived!\a" RESTORE RESET,users[i].name);
				break;
			}
		fflush(stdout);
	}
	return (0);
}